#!/usr/bin/perl

########################################################################
# metaf.pl -- CGI wrapper script to display METAR/TAF conversion results
#
# copyright (c) metaf2xml 2006
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
########################################################################

########################################################################
# some things strictly perl
########################################################################
use strict;
use warnings;

=head1 NAME

metaf.pl -- CGI wrapper script to display METAR/TAF conversion results

=head1 SYNOPSIS

as a standalone script:

  metaf.pl 'type=...&lang=...&format=...&msg=...'

or with HTML:

  <form method="post" action="metaf.pl">
   <input name="type" ... >
   <input name="lang" ... >
   <input name="format" ... >
   <input name="msg" ... >
  </form>

=head1 ARGUMENTS

The script uses the following 4 CGI parameters:

=over

=item B<type>

type of message to be parsed: METAR (default), TAF

=item B<lang>

language for output of script and display of parsed message: de (default), en,
es

=item B<format>

output format: HTML (default), text, or XML (with style sheet reference)

=item B<msg>

message to be parsed or the keyword C<FETCH> and a list of airport ids

=back

=head1 DESCRIPTION

This CGI script provides a web interface to display METAR/TAF conversion
results. It can be used with GET or POST methods. It uses the perl module
C<CGI>.

=cut

use CGI qw/:standard -no_xhtml/;

#
# restrict use of CGI:
#

# uploads are not allowed
$CGI::DISABLE_UPLOADS = 1;

=pod

The length of the body of the POST request (not the length of query string of
the URL) is restricted to 350 bytes.

=cut

$CGI::POST_MAX = 350;

#
# filter line(s) for $icao from $filename
#
sub process_file {
    my ($icao, $filename) = @_;
    my ($found, $response, @arr);

    if (!open(FP, $filename)) {
        return ("$icao - ERROR - " . uc "$filename - $!");
    }
    $found = 0;
    while (<FP>) {
        if (!$found) {
            next unless /^(?:$icao) /;
            $found = 1;
            chomp();
            $response = $_;
        } else {
            my $id;

            if (/^$/) {
                $found = 0;
                $id = ($response =~ /^(....)/)[0];
                @arr = grep {!/^$id / } @arr;
                push @arr, $response;
            } else {
                chomp();
                s/ +/ /g;
                $response .= $_;
            }
        }
    }
    close FP;
    return @arr;
}

#
# fetch line(s) for $icao from $url
#
sub process_url {
    my ($icao, $url) = @_;
    my ($response);

    if (!open(FP, "curl --stderr - $url'$icao' || echo ERROR |")) {
        return ("$icao - ERROR - " . uc $!);
    }
    {
        local $/;
        $response = <FP>;
    }
    close FP;
    if ($response =~ /ERROR/) {
        $response =~ s/[\n\r]+/ /g;
        return ("$icao - ERROR - " . uc "$url - $response");
    }
    $response =~ s/[\n\r]+//g;
    $response =~ s/ +/ /g;
    return $response =~ /<font face=.couri[^>]*?>(?:<pre>)?([^<]+)/go;
}

=pod

If there are any CGI errors the type of error is reported and the script
terminates.

=cut

if (cgi_error()) {
    print header({ status => cgi_error() }) .
        start_html({ title => cgi_error() }) . "\n" .
        b(cgi_error()) . "\n" .
        end_html();
    exit 0;
}

=pod

The following directories and files are used by the script and can be adapted to
local needs by modifying the script (e.g. during install):

=over

=item B<$METAF2XML_BIN>

directory where the C<metaf2xml.pl> is expected (default: C</opt/metaf2xml/bin>)

=item B<$METAF2XML_XSL>

directory where the XSL scripts are located (default: C</opt/metaf2xml/share>)

=item B<$CGI_DATA_DIR>

directory for the local copies of messages (default: C</tmp>)

=item B<$CGI_TMP_DIR>

directory for the temporary XML file (default: C</tmp>)

=back

=cut

########################################################################
# config section, possibly changed by install
########################################################################
my $METAF2XML_BIN = '/opt/metaf2xml/bin';
my $METAF2XML_XSL = '/opt/metaf2xml/share';
my $CGI_DATA_DIR = '/tmp';
my $CGI_LOG_FILE = '/tmp/metaf2xml.log';
my $CGI_TMP_DIR = '/tmp';

=pod

The parameter C<msg> is truncated to 300 characters. Letters are converted to
upper case. Any characters that are not allowed in a message are removed.

=cut

my $msg = param('msg') ? substr(param('msg'), 0, 300) : '';
$msg = uc $msg;
$msg =~ s/[^A-Z0-9.\/\$ +-]//gos;

=pod

The other parameters are checked to have one of the allowed values.

=cut

#
# determine type from request, default (METAR)
#
my $type = param('type');
if (!defined $type || $type ne 'TAF') {
    $type = 'METAR';
}

=pod

If the parameter C<lang> is not set the first allowed value of the HTTP header
C<Accept-Language> is used.

=cut

#
# determine language from request, browser header, default (de)
#
my $lang = param('lang');
if (!defined $lang) {
    my $accept_lang = http('Accept-Language');
    if (defined $accept_lang) {
        for (split(/,/, $accept_lang)) {
            last if $_ =~ /^de/i;
            if ($_ =~ /^en/i) {
                $lang = 'en';
                last;
            }
            if ($_ =~ /^es/i) {
                $lang = 'es';
                last;
            }
        }
    }
}
if (!defined $lang || ($lang ne 'en' && $lang ne 'es')) {
    $lang = 'de';
}

#
# determine format from request, default (html)
#
my $format = param('format');
if (!defined $format || ($format ne 'text' && $format ne 'xml')) {
    $format = 'html';
}

#
# log request
#
if (request_method() && open(LOG, '>>', $CGI_LOG_FILE)) {
    # anonymize IP address, we only want to group the requests
    my $addr = remote_addr();
    $addr =~ s/\.[0-9]+/.x/;
    $addr =~ s/[0-9]+$/x/;
    print LOG gmtime() . ' ' . $addr . ' ' . request_method()
              . " $type $lang $format $msg\n";
    close LOG;
}

=pod

If C<msg> starts with C<METAR> or C<TAF> this message type is used instead of
the parameter C<type>.

=cut

#
# overwrite type if specified in message
#
$type = 'METAR' if $msg =~ /^(?:METAR|SPECI) /;
$type = 'TAF' if $msg =~ /^TAF /;
my $is_taf = $type eq 'TAF' ? '-t ' : '';

#
# translated strings
#
my %trans = (
    title_en => 'METAR/TAF decoder',
    title_de => 'METAR/TAF-bersetzer',
    title_es => 'METAR/TAF traductor',
    header_en => 'METAR/TAF decoder',
    header_de => 'METAR/TAF-bersetzer',
    header_es => 'METAR/TAF traductor',
    source_en => 'A good source for up-to-date METAR and TAF messages is',
    source_de => 'Eine gute Quelle fr aktuelle METAR- und TAF-Meldungen ist',
    source_es => 'Una fuente buena por mensajes actuales de METAR y TAF es',
    source2_en => 'If the keyword "FETCH" and one or more
Airport-Ids are used, messages are fetched from this address
(or a copy on the web server where this page comes from).',
    source2_de => 'Wird das Schlsselwort "FETCH" und ein oder mehrere
Flugplatz-Krzel verwendet, werden Meldungen von dieser Adresse
(oder von einer Kopie auf dem Web-Server, von dem diese Seite kommt) geholt.',
    source2_es => 'Si se usa la palabra clave "FETCH" y uno o mas
ids de la estacin, los mensajes sern trado de esta direccin
(o una copia del servidor del red de donde viene este pgina).',
    example_en => 'examples for messages',
    example_de => 'Beispiele fr Meldungen',
    example_es => 'ejemplos para mensajes',
    message_en => 'message',
    message_de => 'Meldung',
    message_es => 'mensaje',
    type_en => 'is of type',
    type_de => 'ist vom Typ',
    type_es => 'es del typo',
    language_en => 'display in',
    language_de => 'anzeigen in',
    language_es => 'mostrar en',
    format_en   => 'output as',
    format_de   => 'ausgeben als',
    format_es   => 'rendimiento como',
    decode_en   => 'decode',
    decode_de   => 'bersetzen',
    decode_es   => 'traducir',
    disclaimer_en =>
        'Warning: The information on this page may be
out-dated, inaccurate, or both.
It is not suited for use in aviation.',
    disclaimer_de =>
        'Warnung: Die Informationen auf dieser Seite knnten
veraltet, falsch oder beides sein.
Sie sind nicht fr die Verwendung in der Luftfahrt geeignet.',
    disclaimer_es =>
        'Aviso: Las informaciones de esta pgina web puedan ser
antiguas o falsas o los dos.
No son apropiadas para el uso en la aviacin.',
    hint_de =>
"Hinweis: Fr bestimmte Bemerkungen knnen im Moment
nur die Texte (in Englisch) aus der Original-XML-Datei angezeigt werden.",
    hint_en => '',
    hint_es =>
"Indicio: Por algunos comentarios existen solamente los
textos (en ingles) desde el fichero XML original.",
);

#
# save message parameter
#
my $msg_orig = $msg;

=pod

If C<msg> starts with C<FETCH> and is followed by one or more airport ICAO codes
messages for the specified airports are fetched from the local files
C<$CGI_DATA_DIR/{metar,staf,taf}.txt> (if they exist) or the NOAA web site.

=cut

#
# process special contents of message
#
if ($msg =~ /^FETCH((?: [A-Z][A-Z0-9]{3})+)$/) {
    my ($icao, $response, @metar, @staf, @taf);

    $is_taf = '';
    $icao = $1;
    $icao =~ s/ //;

    if (! -f "$CGI_DATA_DIR/metar.txt") {
        my ($metarurl, $stafurl, $tafurl);

        $metarurl = 'http://weather.noaa.gov/cgi-bin/mgetmetar.pl?cccc=';
        $stafurl  = 'http://weather.noaa.gov/cgi-bin/mgetstaf.pl?cccc=';
        $tafurl   = 'http://weather.noaa.gov/cgi-bin/mgettaf.pl?cccc=';
        $icao =~ s/ /+/g;

        @metar = process_url $icao, $metarurl;
        @staf  = process_url $icao, $stafurl;
        @taf   = process_url $icao, $tafurl;
    } else {
        my ($metarfile, $staffile, $taffile);

        $metarfile = "$CGI_DATA_DIR/metar.txt";
        $staffile  = "$CGI_DATA_DIR/staf.txt";
        $taffile   = "$CGI_DATA_DIR/taf.txt";
        $icao =~ s/ /|/g;

        @metar = process_file $icao, $metarfile;
        @staf  = process_file $icao, $staffile;
        @taf   = process_file $icao, $taffile;
    }

    $msg = '';
    for my $id (split /[+|]/, $icao) {
        $msg .= "\nMETAR\n" . join("\n", grep { /^$id/ } @metar);
        $msg .= "\nTAF\n"   . join("\n", grep { /^$id/ } @staf);
        $msg .= "\nTAF\n"   . join("\n", grep { /^$id/ } @taf);
    }
    $msg =~ s/[\n]+/\n/g;
    $msg =~ s/^\n//;
}

if ($format eq 'xml') {
    print "Content-Type: text/xml; charset=ISO-8859-1\n\n";
    open(METAF2XML, "|$METAF2XML_BIN/metaf2xml.pl $is_taf -Xx - -S/metaf-fullhtml.xsl -O'$type $lang $format $msg_orig' 2>&1");
    print METAF2XML $msg;
    close METAF2XML;
    exit 0;
}

print header .
    start_html({ title => $trans{'title_'.$lang},
                 lang  => $lang,
                 dtd   => [ '-//W3C//DTD HTML 4.01 Transitional//EN',
                            'http://www.w3.org/TR/html4/loose.dtd' ],
                 meta  => { robots   => 'noindex,nofollow',
                            keywords =>
                     'metaf2xml,METAR,TAF,XML,aviation,weather,report,forecast',
                          },
                 head  => meta({ http_equiv => 'Content-Type',
                                 content    => 'text/html; charset=ISO-8859-1'})
               }
              ) . "\n" .
    h2($trans{'header_'.$lang}) . "\n" .
    $trans{'source_'.$lang} . ":\n" .
    kbd('http://weather.noaa.gov/weather/coded.html') . br . "\n" .
    $trans{'source2_'.$lang} . p . "\n" .
    start_form({ action => 'metaf.pl' }) .
    table({ border => 0 }, "\n" .
        Tr(td({ colspan => 4 }, $trans{'example_'.$lang} . ':')) . "\n" .
        Tr("\n" .
           td('') . "\n" .
           td({ colspan => 3, style => 'font-family: monospace;' }, "\n" .
              'METAR YUDO 090600Z 00000KT CAVOK 22/15 Q1021' . "\n") . "\n"
        ) . "\n" .
        Tr("\n" .
           td('') . "\n" .
           td({ colspan => 3, style => 'font-family: monospace;' }, "\n" .
              'TAF YUDO 090600Z 090716 VRB03KT 8000 SKC PROB40 TEMPO 0708 0200 +TSRA FM0800 CAVOK' . "\n") . "\n"
        ) . "\n" .
        Tr("\n" .
           td('') . "\n" .
           td({ colspan => 3, style => 'font-family: monospace;' }, "\n" .
              'FETCH SBGL KJFK RJTT' . "\n") . "\n"
        ) . "\n" .
        Tr("\n" .
           td({ nowrap => undef }, $trans{'message_'.$lang} . ':') . "\n" .
           td({ colspan => 3 }, "\n" .
              textfield({ name      => 'msg',
                          size      => 90,
                          maxlength => 300,
                          style     => 'font-family: monospace;',
                          value     => $msg_orig }) . "\n") . "\n"
        ) . "\n" .
        Tr("\n" .
           td({ nowrap => undef }, $trans{'type_'.$lang} . ':') . "\n" .
           td(input({ type => 'radio', name => 'type', value => 'METAR',
                      ($type eq 'METAR' ? 'checked' : '') => undef }
                   ) .
              'METAR') . "\n" .
           td({ colspan => 2 },
              input({ type => 'radio', name => 'type', value => 'TAF',
                      ($type eq 'TAF' ? 'checked' : '') => undef }
                   ) .
              'TAF') . "\n"
        ) . "\n" .
        Tr("\n" .
           td({ nowrap => undef }, $trans{'language_'.$lang} . ':') . "\n" .
           td(input({ type => 'radio', name => 'lang', value => 'de',
                      ($lang eq 'de' ? 'checked' : '') => undef }
                   ) .
              'deutsch') . "\n" .
           td(input({ type => 'radio', name => 'lang', value => 'en',
                      ($lang eq 'en' ? 'checked' : '') => undef }
                   ) .
              'english') . "\n" .
           td(input({ type => 'radio', name => 'lang', value => 'es',
                      ($lang eq 'es' ? 'checked' : '') => undef }
                   ) .
              'espaol') . "\n"
        ) . "\n" .
        Tr("\n" .
           td({ nowrap => undef }, $trans{'format_'.$lang} . ':') . "\n" .
           td({ width => '10%' },
              input({ type => 'radio', name => 'format', value => 'html',
                      ($format eq 'html' ? 'checked' : '') => undef }
                   ) .
              'HTML') . "\n" .
           td({ width => '10%' },
              input({ type => 'radio', name => 'format', value => 'text',
                      ($format eq 'text' ? 'checked' : '') => undef }
                   ) .
              'text') . "\n" .
           td(input({ type => 'radio', name => 'format', value => 'xml',
                      ($format eq 'xml' ? 'checked' : '') => undef }
                   ) .
              'XML (+XSL -> HTML)') . "\n"
        ) . "\n"
    ) . "\n" .
    submit( value => $trans{'decode_'.$lang }) .
    end_form . p . "\n";

if ($msg) {
    my ($xml_out, $response);

    print b($trans{'disclaimer_'.$lang}), p, "\n",
          $trans{'hint_'.$lang}, p, "\n";

    $xml_out = "$CGI_TMP_DIR/metaf2xml-$$.xml";
    open(METAF2XML, "|$METAF2XML_BIN/metaf2xml.pl $is_taf -Xx $xml_out 2>&1");
    print METAF2XML $msg;
    close METAF2XML;
    open(XSLTPROC, "xsltproc --nonet --nowrite --nomkdir --stringparam lang $lang $METAF2XML_XSL/metaf-$format.xsl $xml_out 2>&1 |");
    {
        local $/;
        $response = <XSLTPROC>;
    }
    close XSLTPROC;
    unlink $xml_out;
    if ($format eq 'html') {
        print $response;
    } else {
        print table({ border => 1 }, "\n" .
              Tr("\n" .
                 td("\n" .
                    pre({ style => 'margin: 0;' }, $response)) . "\n") . "\n");
    }
}

print comment('metaf.pl: $Id: metaf.pl,v 1.26 2006/08/09 12:48:26 metaf2xml Exp $'), "\n", p,
      "copyright (c) 2006 metaf2xml @\n",
      a({ href => 'http://metaf2xml.sourceforge.net/' }, "\n" .
      img({ src    =>
              'http://sflogo.sourceforge.net/sflogo.php?group_id=168043&type=1',
            border => 0,
            alt    => 'SourceForge.net Logo' }) . "\n") . "\n" .
      end_html();
